package de.unisaar.jfap.ex4;

import java.util.Collections;
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Iterator wrapper around elements of multiple arrays of type {@code T}.
 *
 * @author Bernd Kiefer, DFKI
 * @author Jörg Steffen, DFKI
 * @param <T> the type of elements returned by this iterator
 */
public class MultArrayIterator<T> implements Iterator<T> {

  private Iterator<T[]> outerIt;
  private Iterator<T> innerIt;

  /**
   * Create a new iterator over the elements of the given arrays.
   *
   * @param arrays the arrays with the elements to iterator over
   */
  @SafeVarargs
  public MultArrayIterator(T[]... arrays) {
    this.outerIt = new ArrayIterator<>(arrays);
    if (this.outerIt.hasNext()) {
      this.innerIt = new ArrayIterator<>(this.outerIt.next());
    } else {
      this.innerIt = Collections.emptyListIterator();
    }
  }

  @Override
  public boolean hasNext() {
    while ((!this.innerIt.hasNext()) && this.outerIt.hasNext()) {
      this.innerIt = new ArrayIterator<>(this.outerIt.next());
    }
    return this.innerIt.hasNext();
  }

  @Override
  public T next() {
    if (!hasNext()) {
      throw new NoSuchElementException();
    }
    return this.innerIt.next();
  }
}
