package de.unisaar.jfap.ex4;

import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Text statistics for words and characters.
 *
 * @author Bernd Kiefer, DFKI
 * @author Jörg Steffen, DFKI
 */
public class TextStatistics {

  private static Pattern wordPattern = Pattern.compile("[A-Za-z]+");

  private Map<String, Integer> wordsCount;
  private Map<Character, Integer> charsCount;

  public TextStatistics() {
    this.wordsCount = new HashMap<>();
    this.charsCount = new HashMap<>();
  }

  /**
   * Update words count with counts from the given string.
   *
   * @param input the input string
   */
  public void countWords(String input) {
    Matcher matcher = wordPattern.matcher(input);
    while (matcher.find()) {
      String word = matcher.group();
      Integer count = this.wordsCount.getOrDefault(word, 0);
      count++;
      this.wordsCount.put(word, count);
    }
  }

  /**
   * Update characters count with count from the given string.
   *
   * @param input the input string
   */
  public void countCharacters(String input) {
    for (char oneChar : input.toCharArray()) {
      Integer count = this.charsCount.getOrDefault(oneChar, 0);
      count++;
      this.charsCount.put(oneChar, count);
    }
  }

  /**
   * Return count for the given word.
   *
   * @param word the word
   * @return the word's count
   */
  public int getWordCount(String word) {
    return this.wordsCount.getOrDefault(word, 0);
  }

  /**
   * Return count for the given character.
   *
   * @param charr the character
   * @return the character's count
   */
  public int getCharacterCount(char charr) {
    return this.charsCount.getOrDefault(charr, 0);
  }

  @Override
  public String toString() {
    StringBuilder result = new StringBuilder();
    for (var oneEntry : this.wordsCount.entrySet()) {
      result.append(String.format("%s: %d%n", oneEntry.getKey(), oneEntry.getValue()));
    }
    result.append(String.format("%n"));
    for (var oneEntry : this.charsCount.entrySet()) {
      result.append(String.format("%s: %d%n", oneEntry.getKey(), oneEntry.getValue()));
    }
    return result.toString();
  }
}
