package de.unisaar.jfap.ex4;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatExceptionOfType;

import java.util.Iterator;
import java.util.NoSuchElementException;
import org.junit.jupiter.api.Test;

class MultArrayIteratorTest {

  @Test
  void testIteratorWithOneArray() {
    String[] stringArray = new String[] {"one", "two", "three"};
    Iterator<String> it = new MultArrayIterator<>(stringArray);
    for (String oneString : stringArray) {
      assertThat(it.hasNext()).isTrue();
      assertThat(it.next()).isEqualTo(oneString);
    }
    assertThat(it.hasNext()).isFalse();
    assertThatExceptionOfType(NoSuchElementException.class).isThrownBy(() -> it.next());
  }

  @Test
  void testIteratorWithTwoArrays() {
    String[] stringArray1 = new String[] {"one", "two", "three"};
    String[] stringArray2 = new String[] {"four", "five", "six"};
    Iterator<String> it = new MultArrayIterator<>(stringArray1, stringArray2);
    for (String[] oneStringArray : new String[][] {stringArray1, stringArray2}) {
      for (String oneString : oneStringArray) {
        assertThat(it.hasNext()).isTrue();
        assertThat(it.next()).isEqualTo(oneString);
      }
    }
    assertThat(it.hasNext()).isFalse();
    assertThatExceptionOfType(NoSuchElementException.class).isThrownBy(() -> it.next());
  }

  void testIteratorEmpty() {
    String[] stringArray = new String[0];
    Iterator<String> it = new MultArrayIterator<>(stringArray);
    assertThat(it.hasNext()).isFalse();
    assertThatExceptionOfType(NoSuchElementException.class).isThrownBy(() -> it.next());
  }
}
